#include<string.h>
#include<stdlib.h>
#include"argon.h"

#include"GL_helper.h"
#include"HD_helper.h"
#include"calibration.h"

#include<unistd.h>
#include<time.h>

t_state *state;
t_parameters pars;

double forceScale=1000;

/*******************************************************************************
 Graphics main loop function.
*******************************************************************************/
void displayFunction(void)
{
    // Setup model transformations.
    glMatrixMode(GL_MODELVIEW); 
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();

    setupGraphicsState();

    // Draw the fixed sphere.
    float atomColor[4] = {.2, .8, .8, .8};
    float selAtomColor[4] = {.8, .2, .2, .8};
    float actAtomColor[4] = {.2, .8, .2, .8};
    float lockAtomColor[4] = {.2, .5, .5, .8};


    HapticDisplayState stateHD;

    /* Obtain a thread-safe copy of the current haptic display state. */
    hdScheduleSynchronous(copyHapticDisplayState, &stateHD, HD_DEFAULT_SCHEDULER_PRIORITY);

	if(activeAtom>=0) {
		for(int i=0; i<state->N; i++) {
			if(i==activeAtom) {
				drawSphere((double*)&(state->r[i]), actAtomColor, atomRadius*1.1);
			} else {
				if(state->locked[i]) {
					drawSphere((double*)&(state->r[i]), lockAtomColor, atomRadius);
				} else {
					drawSphere((double*)&(state->r[i]), atomColor, atomRadius);
				}

			}
		}
	} else {
		t_coord cursor={0,0,0,0};
		getModelCoord(&cursor, (stateHD.position));
		for(int i=0; i<state->N; i++) {
			if(dist(state->r[i], cursor)<atomRadius) {
				drawSphere((double*)&(state->r[i]), selAtomColor, atomRadius*1.1);
			} else {
				if(state->locked[i]) {
					drawSphere((double*)&(state->r[i]), lockAtomColor, atomRadius);
				} else {
					drawSphere((double*)&(state->r[i]), atomColor, atomRadius);
				}
			}
		}
	}

	drawCursor(&stateHD);

	glPopMatrix();

	glutSwapBuffers();                      

	updateWorkspace();
}


void redrawTimer(int msec) {
	static int lastButton1=0;
	static int lastButton2=0;

	one_step(state, &pars);
	HapticDisplayState stateHD;

    /* Obtain a thread-safe copy of the current haptic display state. */
    hdScheduleSynchronous(copyHapticDisplayState, &stateHD, HD_DEFAULT_SCHEDULER_PRIORITY);
	
	t_coord cursor={0,0,0,0};
	getModelCoord(&cursor, (stateHD.position));

	if(stateHD.button1==0 && stateHD.button2==0) {
		if(lastButton1 && activeAtom>=0)	{ // Let go an atom, if one was held
			hdScheduleSynchronous(resetHapticAnchor, 0, HD_DEFAULT_SCHEDULER_PRIORITY);
			state->dF[activeAtom].x=0;
			state->dF[activeAtom].y=0;
			state->dF[activeAtom].z=0;
		}

		activeAtom=-1;
	} else if(activeAtom==-1) {
		for(int i=0; i<state->N; i++) {
			if(dist(state->r[i], cursor)<atomRadius) { // User pressed a button near an atom -- activate.
				printf("activated: %d\n", i);
				activeAtom=i;
			}
		}
	}

	if(stateHD.button1) {
		if(activeAtom>=0) { // Atom is being dragged, we have to update a force in the model.
			if(lastButton1) {
				t_coord force;
				t_coord pos;
				t_coord anchor;

				getModelCoord(&anchor, (stateHD.anchor));
				getModelCoord(&pos, (stateHD.position));
				getModelCoord(&force, (stateHD.force));


				state->dF[activeAtom].x=(pos.x-anchor.x)*forceScale;
				state->dF[activeAtom].y=(pos.y-anchor.y)*forceScale;
				state->dF[activeAtom].z=(pos.z-anchor.z)*forceScale;
			}

			hduVector3Dd pos;
			getHapticCoord(pos, state->r[activeAtom]);
			hdScheduleSynchronous(setHapticAnchor, &(pos), HD_DEFAULT_SCHEDULER_PRIORITY);
		}
	} else if(stateHD.button2) {
		if(activeAtom>=0 && !lastButton2) {  // toggle atom freeze
			state->locked[activeAtom]= state->locked[activeAtom]==1 ? 0 : 1;
		}
	}

	lastButton1=stateHD.button1;
	lastButton2=stateHD.button2;

	glutTimerFunc(msec, redrawTimer, msec);
	glutPostRedisplay();
}

int main(int argc, char **argv)
{

	if(argc<2) {
		printf("Usage: %s <input file>\n", argv[0]);
		exit(-1);
	}

	srand(time(NULL)%getpid()+20);

	char *fname_in=argv[1];


	load_parameters(&pars, fname_in);

	atomRadius=pars.R/2;
		
	state=alloc_state(pars.N);

	init_state(state, &pars);
	compute_forces(state, &pars);

	for(int i=0; i<state->N-1; i++) {
		state->locked[i]=1;
	}

	initHD();
	
	initGlut(argc, argv);

    glutDisplayFunc(displayFunction); 

	glutTimerFunc(5, redrawTimer, 5);
    atexit(exitHandler);

	glutMainLoop();
}
